@@ -1,4 +1,5 @@
 require 'abstract_unit'
+require 'action_view'
 require 'action_controller'
 
 class I18nTestMailer < ActionMailer::Base
@@ -9,22 +10,22 @@ class I18nTestMailer < ActionMailer::Base
   def mail_with_i18n_subject(recipient)
     @recipient  = recipient
     I18n.locale = :de
-    mail(:to => recipient, :subject => "#{I18n.t :email_subject} #{recipient}",
-      :from => "system@loudthinking.com", :date => Time.local(2004, 12, 12))
+    mail(to: recipient, subject: "#{I18n.t :email_subject} #{recipient}",
+      from: "system@loudthinking.com", date: Time.local(2004, 12, 12))
   end
 end
 
 class TestController < ActionController::Base
   def send_mail
     I18nTestMailer.mail_with_i18n_subject("test@localhost").deliver
-    render :text => 'Mail sent'
+    render text: 'Mail sent'
   end
 end
 
 class ActionMailerI18nWithControllerTest < ActionDispatch::IntegrationTest
   Routes = ActionDispatch::Routing::RouteSet.new
   Routes.draw do
-    match ':controller(/:action(/:id))'
+    get ':controller(/:action(/:id))'
   end
 
   def app
@@ -32,7 +33,7 @@ class ActionMailerI18nWithControllerTest < ActionDispatch::IntegrationTest
   end
 
   def setup
-    I18n.backend.store_translations('de', :email_subject => '[Signed up] Welcome')
+    I18n.backend.store_translations('de', email_subject: '[Signed up] Welcome')
   end
 
   def teardown
