@@ -1,7 +1,32 @@
 require 'abstract_unit'
 
-# FIXME: crashes Ruby 1.9
-class FilterTest < Test::Unit::TestCase
+class ActionController::Base
+  class << self
+    %w(append_around_filter prepend_after_filter prepend_around_filter prepend_before_filter skip_after_filter skip_before_filter skip_filter).each do |pending|
+      define_method(pending) do |*args|
+        $stderr.puts "#{pending} unimplemented: #{args.inspect}"
+      end unless method_defined?(pending)
+    end
+
+    def before_filters
+      filters = _process_action_callbacks.select { |c| c.kind == :before }
+      filters.map! { |c| c.raw_filter }
+    end
+  end
+
+  def assigns(key = nil)
+    assigns = {}
+    instance_variables.each do |ivar|
+      next if ActionController::Base.protected_instance_variables.include?(ivar)
+      assigns[ivar[1..-1]] = instance_variable_get(ivar)
+    end
+
+    key.nil? ? assigns : assigns[key.to_s]
+  end
+end
+
+class FilterTest < ActionController::TestCase
+
   class TestController < ActionController::Base
     before_filter :ensure_login
     after_filter  :clean_up
@@ -53,7 +78,8 @@ class FilterTest < Test::Unit::TestCase
   end
 
   class RenderingController < ActionController::Base
-    before_filter :render_something_else
+    before_filter :before_filter_rendering
+    after_filter :unreached_after_filter
 
     def show
       @ran_action = true
@@ -61,9 +87,59 @@ class FilterTest < Test::Unit::TestCase
     end
 
     private
-      def render_something_else
+      def before_filter_rendering
+        @ran_filter ||= []
+        @ran_filter << "before_filter_rendering"
         render :inline => "something else"
       end
+
+      def unreached_after_filter
+        @ran_filter << "unreached_after_filter_after_render"
+      end
+  end
+
+  class RenderingForPrependAfterFilterController < RenderingController
+    prepend_after_filter :unreached_prepend_after_filter
+
+    private
+      def unreached_prepend_after_filter
+        @ran_filter << "unreached_preprend_after_filter_after_render"
+      end
+  end
+
+  class BeforeFilterRedirectionController < ActionController::Base
+    before_filter :before_filter_redirects
+    after_filter :unreached_after_filter
+
+    def show
+      @ran_action = true
+      render :inline => "ran show action"
+    end
+
+    def target_of_redirection
+      @ran_target_of_redirection = true
+      render :inline => "ran target_of_redirection action"
+    end
+
+    private
+      def before_filter_redirects
+        @ran_filter ||= []
+        @ran_filter << "before_filter_redirects"
+        redirect_to(:action => 'target_of_redirection')
+      end
+
+      def unreached_after_filter
+        @ran_filter << "unreached_after_filter_after_redirection"
+      end
+  end
+
+  class BeforeFilterRedirectionForPrependAfterFilterController < BeforeFilterRedirectionController
+    prepend_after_filter :unreached_prepend_after_filter_after_redirection
+
+    private
+      def unreached_prepend_after_filter_after_redirection
+        @ran_filter << "unreached_prepend_after_filter_after_redirection"
+      end
   end
 
   class ConditionalFilterController < ActionController::Base
@@ -89,8 +165,6 @@ class FilterTest < Test::Unit::TestCase
         @ran_filter ||= []
         @ran_filter << "clean_up_tmp"
       end
-
-      def rescue_action(e) raise(e) end
   end
 
   class ConditionalCollectionFilterController < ConditionalFilterController
@@ -111,15 +185,15 @@ class FilterTest < Test::Unit::TestCase
   end
 
   class OnlyConditionProcController < ConditionalFilterController
-    before_filter(:only => :show) {|c| c.assigns["ran_proc_filter"] = true }
+    before_filter(:only => :show) {|c| c.instance_variable_set(:"@ran_proc_filter", true) }
   end
 
   class ExceptConditionProcController < ConditionalFilterController
-    before_filter(:except => :show_without_filter) {|c| c.assigns["ran_proc_filter"] = true }
+    before_filter(:except => :show_without_filter) {|c| c.instance_variable_set(:"@ran_proc_filter", true) }
   end
 
   class ConditionalClassFilter
-    def self.filter(controller) controller.assigns["ran_class_filter"] = true end
+    def self.before(controller) controller.instance_variable_set(:"@ran_class_filter", true) end
   end
 
   class OnlyConditionClassController < ConditionalFilterController
@@ -131,7 +205,11 @@ class FilterTest < Test::Unit::TestCase
   end
 
   class AnomolousYetValidConditionController < ConditionalFilterController
-    before_filter(ConditionalClassFilter, :ensure_login, Proc.new {|c| c.assigns["ran_proc_filter1"] = true }, :except => :show_without_filter) { |c| c.assigns["ran_proc_filter2"] = true}
+    before_filter(ConditionalClassFilter, :ensure_login, Proc.new {|c| c.instance_variable_set(:"@ran_proc_filter1", true)}, :except => :show_without_filter) { |c| c.instance_variable_set(:"@ran_proc_filter2", true)}
+  end
+
+  class OnlyConditionalOptionsFilter < ConditionalFilterController
+    before_filter :ensure_login, :only => :index, :if => Proc.new {|c| c.instance_variable_set(:"@ran_conditional_index_proc", true) }
   end
 
   class ConditionalOptionsFilter < ConditionalFilterController
@@ -139,12 +217,16 @@ class FilterTest < Test::Unit::TestCase
     before_filter :clean_up_tmp, :if => Proc.new { |c| false }
   end
 
-  class EmptyFilterChainController < TestController
-    self.filter_chain.clear
-    def show
-      @action_executed = true
-      render :text => "yawp!"
-    end
+  class ConditionalOptionsSkipFilter < ConditionalFilterController
+    before_filter :ensure_login
+    before_filter :clean_up_tmp
+
+    skip_before_filter :ensure_login, if: -> { false }
+    skip_before_filter :clean_up_tmp, if: -> { true }
+  end
+
+  class ClassController < ConditionalFilterController
+    before_filter ConditionalClassFilter
   end
 
   class PrependingController < TestController
@@ -165,16 +247,21 @@ class FilterTest < Test::Unit::TestCase
     def index
       render :text => 'ok'
     end
-    
+
     def public
+      render :text => 'ok'
     end
   end
-  
+
   class SkippingAndReorderingController < TestController
     skip_before_filter :ensure_login
     before_filter :find_record
     before_filter :ensure_login
 
+    def index
+      render :text => 'ok'
+    end
+
     private
       def find_record
         @ran_filter ||= []
@@ -204,37 +291,42 @@ class FilterTest < Test::Unit::TestCase
   end
 
   class ConditionalParentOfConditionalSkippingController < ConditionalFilterController
-    before_filter :conditional_in_parent, :only => [:show, :another_action]
-    after_filter  :conditional_in_parent, :only => [:show, :another_action]
+    before_filter :conditional_in_parent_before, :only => [:show, :another_action]
+    after_filter  :conditional_in_parent_after, :only => [:show, :another_action]
 
     private
 
-      def conditional_in_parent
+      def conditional_in_parent_before
         @ran_filter ||= []
-        @ran_filter << 'conditional_in_parent'
+        @ran_filter << 'conditional_in_parent_before'
+      end
+
+      def conditional_in_parent_after
+        @ran_filter ||= []
+        @ran_filter << 'conditional_in_parent_after'
       end
   end
 
   class ChildOfConditionalParentController < ConditionalParentOfConditionalSkippingController
-    skip_before_filter :conditional_in_parent, :only => :another_action
-    skip_after_filter  :conditional_in_parent, :only => :another_action
+    skip_before_filter :conditional_in_parent_before, :only => :another_action
+    skip_after_filter  :conditional_in_parent_after, :only => :another_action
   end
 
   class AnotherChildOfConditionalParentController < ConditionalParentOfConditionalSkippingController
-    skip_before_filter :conditional_in_parent, :only => :show
+    skip_before_filter :conditional_in_parent_before, :only => :show
   end
 
   class ProcController < PrependingController
-    before_filter(proc { |c| c.assigns["ran_proc_filter"] = true })
+    before_filter(proc { |c| c.instance_variable_set(:"@ran_proc_filter", true) })
   end
 
   class ImplicitProcController < PrependingController
-    before_filter { |c| c.assigns["ran_proc_filter"] = true }
+    before_filter { |c| c.instance_variable_set(:"@ran_proc_filter", true) }
   end
 
   class AuditFilter
-    def self.filter(controller)
-      controller.assigns["was_audited"] = true
+    def self.before(controller)
+      controller.instance_variable_set(:"@was_audited", true)
     end
   end
 
@@ -242,14 +334,20 @@ class FilterTest < Test::Unit::TestCase
     def before(controller)
       @execution_log = "before"
       controller.class.execution_log << " before aroundfilter " if controller.respond_to? :execution_log
-      controller.assigns["before_ran"] = true
+      controller.instance_variable_set(:"@before_ran", true)
     end
 
     def after(controller)
-      controller.assigns["execution_log"] = @execution_log + " and after"
-      controller.assigns["after_ran"] = true
+      controller.instance_variable_set(:"@execution_log", @execution_log + " and after")
+      controller.instance_variable_set(:"@after_ran", true)
       controller.class.execution_log << " after aroundfilter " if controller.respond_to? :execution_log
     end
+
+    def around(controller)
+      before(controller)
+      yield
+      after(controller)
+    end
   end
 
   class AppendedAroundFilter
@@ -260,6 +358,12 @@ class FilterTest < Test::Unit::TestCase
     def after(controller)
       controller.class.execution_log << " after appended aroundfilter "
     end
+
+    def around(controller)
+      before(controller)
+      yield
+      after(controller)
+    end
   end
 
   class AuditController < ActionController::Base
@@ -287,6 +391,7 @@ class FilterTest < Test::Unit::TestCase
 
     def initialize
       @@execution_log = ""
+      super()
     end
 
     before_filter { |c| c.class.execution_log << " before procfilter "  }
@@ -360,12 +465,10 @@ class FilterTest < Test::Unit::TestCase
   class ErrorToRescue < Exception; end
 
   class RescuingAroundFilterWithBlock
-    def filter(controller)
-      begin
-        yield
-      rescue ErrorToRescue => ex
-        controller.send! :render, :text => "I rescued this: #{ex.inspect}"
-      end
+    def around(controller)
+      yield
+    rescue ErrorToRescue => ex
+      controller.__send__ :render, :text => "I rescued this: #{ex.inspect}"
     end
   end
 
@@ -375,11 +478,6 @@ class FilterTest < Test::Unit::TestCase
     def show
       raise ErrorToRescue.new("Something made the bad noise.")
     end
-
-  private
-    def rescue_action(exception)
-      raise exception
-    end
   end
 
   class NonYieldingAroundFilterController < ActionController::Base
@@ -393,9 +491,6 @@ class FilterTest < Test::Unit::TestCase
       render :inline => "index"
     end
 
-    #make sure the controller complains
-    def rescue_action(e); raise e; end
-
     private
 
       def filter_one
@@ -408,7 +503,7 @@ class FilterTest < Test::Unit::TestCase
       end
 
       def non_yielding_filter
-        @filters  << "zomg it didn't yield"
+        @filters  << "it didn't yield"
         @filter_return_value
       end
 
@@ -418,6 +513,21 @@ class FilterTest < Test::Unit::TestCase
 
   end
 
+  class ImplicitActionsController < ActionController::Base
+    before_filter :find_only, :only => :edit
+    before_filter :find_except, :except => :edit
+
+    private
+
+    def find_only
+      @only = 'Only'
+    end
+
+    def find_except
+      @except = 'Except'
+    end
+  end
+
   def test_non_yielding_around_filters_not_returning_false_do_not_raise
     controller = NonYieldingAroundFilterController.new
     controller.instance_variable_set "@filter_return_value", true
@@ -438,19 +548,14 @@ class FilterTest < Test::Unit::TestCase
     controller = NonYieldingAroundFilterController.new
     controller.instance_variable_set "@filter_return_value", false
     test_process(controller, "index")
-    assert_equal ["filter_one", "zomg it didn't yield"], controller.assigns['filters']
+    assert_equal ["filter_one", "it didn't yield"], controller.assigns['filters']
   end
 
   def test_after_filters_are_not_run_if_around_filter_does_not_yield
     controller = NonYieldingAroundFilterController.new
     controller.instance_variable_set "@filter_return_value", true
     test_process(controller, "index")
-    assert_equal ["filter_one", "zomg it didn't yield"], controller.assigns['filters']
-  end
-
-  def test_empty_filter_chain
-    assert_equal 0, EmptyFilterChainController.filter_chain.size
-    assert test_process(EmptyFilterChainController).template.assigns['action_executed']
+    assert_equal ["filter_one", "it didn't yield"], controller.assigns['filters']
   end
 
   def test_added_filter_to_inheritance_graph
@@ -466,101 +571,170 @@ class FilterTest < Test::Unit::TestCase
   end
 
   def test_running_filters
-    assert_equal %w( wonderful_life ensure_login ), test_process(PrependingController).template.assigns["ran_filter"]
+    test_process(PrependingController)
+    assert_equal %w( wonderful_life ensure_login ), assigns["ran_filter"]
   end
 
   def test_running_filters_with_proc
-    assert test_process(ProcController).template.assigns["ran_proc_filter"]
+    test_process(ProcController)
+    assert assigns["ran_proc_filter"]
   end
 
   def test_running_filters_with_implicit_proc
-    assert test_process(ImplicitProcController).template.assigns["ran_proc_filter"]
+    test_process(ImplicitProcController)
+    assert assigns["ran_proc_filter"]
   end
 
   def test_running_filters_with_class
-    assert test_process(AuditController).template.assigns["was_audited"]
+    test_process(AuditController)
+    assert assigns["was_audited"]
   end
 
   def test_running_anomolous_yet_valid_condition_filters
-    response = test_process(AnomolousYetValidConditionController)
-    assert_equal %w( ensure_login ), response.template.assigns["ran_filter"]
-    assert response.template.assigns["ran_class_filter"]
-    assert response.template.assigns["ran_proc_filter1"]
-    assert response.template.assigns["ran_proc_filter2"]
+    test_process(AnomolousYetValidConditionController)
+    assert_equal %w( ensure_login ), assigns["ran_filter"]
+    assert assigns["ran_class_filter"]
+    assert assigns["ran_proc_filter1"]
+    assert assigns["ran_proc_filter2"]
 
-    response = test_process(AnomolousYetValidConditionController, "show_without_filter")
-    assert_equal nil, response.template.assigns["ran_filter"]
-    assert !response.template.assigns["ran_class_filter"]
-    assert !response.template.assigns["ran_proc_filter1"]
-    assert !response.template.assigns["ran_proc_filter2"]
+    test_process(AnomolousYetValidConditionController, "show_without_filter")
+    assert_nil assigns["ran_filter"]
+    assert !assigns["ran_class_filter"]
+    assert !assigns["ran_proc_filter1"]
+    assert !assigns["ran_proc_filter2"]
   end
 
   def test_running_conditional_options
-    response = test_process(ConditionalOptionsFilter)
-    assert_equal %w( ensure_login ), response.template.assigns["ran_filter"]
+    test_process(ConditionalOptionsFilter)
+    assert_equal %w( ensure_login ), assigns["ran_filter"]
+  end
+
+  def test_running_conditional_skip_options
+    test_process(ConditionalOptionsSkipFilter)
+    assert_equal %w( ensure_login ), assigns["ran_filter"]
+  end
+
+  def test_skipping_class_filters
+    test_process(ClassController)
+    assert_equal true, assigns["ran_class_filter"]
+
+    skipping_class_controller = Class.new(ClassController) do
+      skip_before_filter ConditionalClassFilter
+    end
+
+    test_process(skipping_class_controller)
+    assert_nil assigns['ran_class_filter']
   end
 
   def test_running_collection_condition_filters
-    assert_equal %w( ensure_login ), test_process(ConditionalCollectionFilterController).template.assigns["ran_filter"]
-    assert_equal nil, test_process(ConditionalCollectionFilterController, "show_without_filter").template.assigns["ran_filter"]
-    assert_equal nil, test_process(ConditionalCollectionFilterController, "another_action").template.assigns["ran_filter"]
+    test_process(ConditionalCollectionFilterController)
+    assert_equal %w( ensure_login ), assigns["ran_filter"]
+    test_process(ConditionalCollectionFilterController, "show_without_filter")
+    assert_nil assigns["ran_filter"]
+    test_process(ConditionalCollectionFilterController, "another_action")
+    assert_nil assigns["ran_filter"]
   end
 
   def test_running_only_condition_filters
-    assert_equal %w( ensure_login ), test_process(OnlyConditionSymController).template.assigns["ran_filter"]
-    assert_equal nil, test_process(OnlyConditionSymController, "show_without_filter").template.assigns["ran_filter"]
+    test_process(OnlyConditionSymController)
+    assert_equal %w( ensure_login ), assigns["ran_filter"]
+    test_process(OnlyConditionSymController, "show_without_filter")
+    assert_nil assigns["ran_filter"]
 
-    assert test_process(OnlyConditionProcController).template.assigns["ran_proc_filter"]
-    assert !test_process(OnlyConditionProcController, "show_without_filter").template.assigns["ran_proc_filter"]
+    test_process(OnlyConditionProcController)
+    assert assigns["ran_proc_filter"]
+    test_process(OnlyConditionProcController, "show_without_filter")
+    assert !assigns["ran_proc_filter"]
 
-    assert test_process(OnlyConditionClassController).template.assigns["ran_class_filter"]
-    assert !test_process(OnlyConditionClassController, "show_without_filter").template.assigns["ran_class_filter"]
+    test_process(OnlyConditionClassController)
+    assert assigns["ran_class_filter"]
+    test_process(OnlyConditionClassController, "show_without_filter")
+    assert !assigns["ran_class_filter"]
   end
 
   def test_running_except_condition_filters
-    assert_equal %w( ensure_login ), test_process(ExceptConditionSymController).template.assigns["ran_filter"]
-    assert_equal nil, test_process(ExceptConditionSymController, "show_without_filter").template.assigns["ran_filter"]
+    test_process(ExceptConditionSymController)
+    assert_equal %w( ensure_login ), assigns["ran_filter"]
+    test_process(ExceptConditionSymController, "show_without_filter")
+    assert_nil assigns["ran_filter"]
 
-    assert test_process(ExceptConditionProcController).template.assigns["ran_proc_filter"]
-    assert !test_process(ExceptConditionProcController, "show_without_filter").template.assigns["ran_proc_filter"]
+    test_process(ExceptConditionProcController)
+    assert assigns["ran_proc_filter"]
+    test_process(ExceptConditionProcController, "show_without_filter")
+    assert !assigns["ran_proc_filter"]
 
-    assert test_process(ExceptConditionClassController).template.assigns["ran_class_filter"]
-    assert !test_process(ExceptConditionClassController, "show_without_filter").template.assigns["ran_class_filter"]
+    test_process(ExceptConditionClassController)
+    assert assigns["ran_class_filter"]
+    test_process(ExceptConditionClassController, "show_without_filter")
+    assert !assigns["ran_class_filter"]
+  end
+
+  def test_running_only_condition_and_conditional_options
+    test_process(OnlyConditionalOptionsFilter, "show")
+    assert_not assigns["ran_conditional_index_proc"]
   end
 
   def test_running_before_and_after_condition_filters
-    assert_equal %w( ensure_login clean_up_tmp), test_process(BeforeAndAfterConditionController).template.assigns["ran_filter"]
-    assert_equal nil, test_process(BeforeAndAfterConditionController, "show_without_filter").template.assigns["ran_filter"]
+    test_process(BeforeAndAfterConditionController)
+    assert_equal %w( ensure_login clean_up_tmp), assigns["ran_filter"]
+    test_process(BeforeAndAfterConditionController, "show_without_filter")
+    assert_nil assigns["ran_filter"]
   end
 
   def test_around_filter
-    controller = test_process(AroundFilterController)
-    assert controller.template.assigns["before_ran"]
-    assert controller.template.assigns["after_ran"]
+    test_process(AroundFilterController)
+    assert assigns["before_ran"]
+    assert assigns["after_ran"]
   end
 
   def test_before_after_class_filter
-    controller = test_process(BeforeAfterClassFilterController)
-    assert controller.template.assigns["before_ran"]
-    assert controller.template.assigns["after_ran"]
+    test_process(BeforeAfterClassFilterController)
+    assert assigns["before_ran"]
+    assert assigns["after_ran"]
   end
 
   def test_having_properties_in_around_filter
-    controller = test_process(AroundFilterController)
-    assert_equal "before and after", controller.template.assigns["execution_log"]
+    test_process(AroundFilterController)
+    assert_equal "before and after", assigns["execution_log"]
   end
 
   def test_prepending_and_appending_around_filter
-    controller = test_process(MixedFilterController)
+    test_process(MixedFilterController)
     assert_equal " before aroundfilter  before procfilter  before appended aroundfilter " +
-                 " after appended aroundfilter  after aroundfilter  after procfilter ",
+                 " after appended aroundfilter  after procfilter  after aroundfilter ",
                  MixedFilterController.execution_log
   end
 
   def test_rendering_breaks_filtering_chain
     response = test_process(RenderingController)
     assert_equal "something else", response.body
-    assert !response.template.assigns["ran_action"]
+    assert !assigns["ran_action"]
+  end
+
+  def test_before_filter_rendering_breaks_filtering_chain_for_after_filter
+    test_process(RenderingController)
+    assert_equal %w( before_filter_rendering ), assigns["ran_filter"]
+    assert !assigns["ran_action"]
+  end
+
+  def test_before_filter_redirects_breaks_filtering_chain_for_after_filter
+    test_process(BeforeFilterRedirectionController)
+    assert_response :redirect
+    assert_equal "http://test.host/filter_test/before_filter_redirection/target_of_redirection", redirect_to_url
+    assert_equal %w( before_filter_redirects ), assigns["ran_filter"]
+  end
+
+  def test_before_filter_rendering_breaks_filtering_chain_for_preprend_after_filter
+    test_process(RenderingForPrependAfterFilterController)
+    assert_equal %w( before_filter_rendering ), assigns["ran_filter"]
+    assert !assigns["ran_action"]
+  end
+
+  def test_before_filter_redirects_breaks_filtering_chain_for_preprend_after_filter
+    test_process(BeforeFilterRedirectionForPrependAfterFilterController)
+    assert_response :redirect
+    assert_equal "http://test.host/filter_test/before_filter_redirection_for_prepend_after_filter/target_of_redirection", redirect_to_url
+    assert_equal %w( before_filter_redirects ), assigns["ran_filter"]
   end
 
   def test_filters_with_mixed_specialization_run_in_order
@@ -579,47 +753,59 @@ class FilterTest < Test::Unit::TestCase
     %w(foo bar baz).each do |action|
       request = ActionController::TestRequest.new
       request.query_parameters[:choose] = action
-      response = DynamicDispatchController.process(request, ActionController::TestResponse.new)
+      response = DynamicDispatchController.action(action).call(request.env).last
       assert_equal action, response.body
     end
   end
 
   def test_running_prepended_before_and_after_filter
-    assert_equal 3, PrependingBeforeAndAfterController.filter_chain.length
-    response = test_process(PrependingBeforeAndAfterController)
-    assert_equal %w( before_all between_before_all_and_after_all after_all ), response.template.assigns["ran_filter"]
+    test_process(PrependingBeforeAndAfterController)
+    assert_equal %w( before_all between_before_all_and_after_all after_all ), assigns["ran_filter"]
   end
-  
+
   def test_skipping_and_limiting_controller
-    assert_equal %w( ensure_login ), test_process(SkippingAndLimitedController, "index").template.assigns["ran_filter"]
-    assert_nil test_process(SkippingAndLimitedController, "public").template.assigns["ran_filter"]
+    test_process(SkippingAndLimitedController, "index")
+    assert_equal %w( ensure_login ), assigns["ran_filter"]
+    test_process(SkippingAndLimitedController, "public")
+    assert_nil assigns["ran_filter"]
   end
 
   def test_skipping_and_reordering_controller
-    assert_equal %w( find_record ensure_login ), test_process(SkippingAndReorderingController, "index").template.assigns["ran_filter"]
+    test_process(SkippingAndReorderingController, "index")
+    assert_equal %w( find_record ensure_login ), assigns["ran_filter"]
   end
 
   def test_conditional_skipping_of_filters
-    assert_nil test_process(ConditionalSkippingController, "login").template.assigns["ran_filter"]
-    assert_equal %w( ensure_login find_user ), test_process(ConditionalSkippingController, "change_password").template.assigns["ran_filter"]
+    test_process(ConditionalSkippingController, "login")
+    assert_nil assigns["ran_filter"]
+    test_process(ConditionalSkippingController, "change_password")
+    assert_equal %w( ensure_login find_user ), assigns["ran_filter"]
 
-    assert_nil test_process(ConditionalSkippingController, "login").template.controller.instance_variable_get("@ran_after_filter")
-    assert_equal %w( clean_up ), test_process(ConditionalSkippingController, "change_password").template.controller.instance_variable_get("@ran_after_filter")
+    test_process(ConditionalSkippingController, "login")
+    assert !@controller.instance_variable_defined?("@ran_after_filter")
+    test_process(ConditionalSkippingController, "change_password")
+    assert_equal %w( clean_up ), @controller.instance_variable_get("@ran_after_filter")
   end
 
   def test_conditional_skipping_of_filters_when_parent_filter_is_also_conditional
-    assert_equal %w( conditional_in_parent conditional_in_parent ), test_process(ChildOfConditionalParentController).template.assigns['ran_filter']
-    assert_nil test_process(ChildOfConditionalParentController, 'another_action').template.assigns['ran_filter']
+    test_process(ChildOfConditionalParentController)
+    assert_equal %w( conditional_in_parent_before conditional_in_parent_after ), assigns['ran_filter']
+    test_process(ChildOfConditionalParentController, 'another_action')
+    assert_nil assigns['ran_filter']
   end
 
   def test_condition_skipping_of_filters_when_siblings_also_have_conditions
-    assert_equal %w( conditional_in_parent conditional_in_parent ), test_process(ChildOfConditionalParentController).template.assigns['ran_filter'], "1"
-    assert_equal nil, test_process(AnotherChildOfConditionalParentController).template.assigns['ran_filter']
-    assert_equal %w( conditional_in_parent conditional_in_parent ), test_process(ChildOfConditionalParentController).template.assigns['ran_filter']
+    test_process(ChildOfConditionalParentController)
+    assert_equal %w( conditional_in_parent_before conditional_in_parent_after ), assigns['ran_filter']
+    test_process(AnotherChildOfConditionalParentController)
+    assert_equal %w( conditional_in_parent_after ), assigns['ran_filter']
+    test_process(ChildOfConditionalParentController)
+    assert_equal %w( conditional_in_parent_before conditional_in_parent_after ), assigns['ran_filter']
   end
 
   def test_changing_the_requirements
-    assert_equal nil, test_process(ChangingTheRequirementsController, "go_wild").template.assigns['ran_filter']
+    test_process(ChangingTheRequirementsController, "go_wild")
+    assert_nil assigns['ran_filter']
   end
 
   def test_a_rescuing_around_filter
@@ -632,19 +818,29 @@ class FilterTest < Test::Unit::TestCase
     assert_equal("I rescued this: #<FilterTest::ErrorToRescue: Something made the bad noise.>", response.body)
   end
 
+  def test_filters_obey_only_and_except_for_implicit_actions
+    test_process(ImplicitActionsController, 'show')
+    assert_equal 'Except', assigns(:except)
+    assert_nil assigns(:only)
+    assert_equal 'show', response.body
+
+    test_process(ImplicitActionsController, 'edit')
+    assert_equal 'Only', assigns(:only)
+    assert_nil assigns(:except)
+    assert_equal 'edit', response.body
+  end
+
   private
     def test_process(controller, action = "show")
-      request = ActionController::TestRequest.new
-      request.action = action
-      controller.process(request, ActionController::TestResponse.new)
+      @controller = controller.is_a?(Class) ? controller.new : controller
+      @request    = ActionController::TestRequest.new
+      @response   = ActionController::TestResponse.new
+
+      process(action)
     end
 end
 
-
-
 class PostsController < ActionController::Base
-  def rescue_action(e); raise e; end
-
   module AroundExceptions
     class Error < StandardError ; end
     class Before < Error ; end
@@ -682,18 +878,18 @@ class ControllerWithSymbolAsFilter < PostsController
 
     def without_exception
       # Do stuff...
-      1 + 1
+      wtf = 1 + 1
 
       yield
 
       # Do stuff...
-      1 + 1
+      wtf += 1
     end
 end
 
 class ControllerWithFilterClass < PostsController
   class YieldingFilter < DefaultFilter
-    def self.filter(controller)
+    def self.around(controller)
       yield
       raise After
     end
@@ -704,7 +900,7 @@ end
 
 class ControllerWithFilterInstance < PostsController
   class YieldingFilter < DefaultFilter
-    def filter(controller)
+    def around(controller)
       yield
       raise After
     end
@@ -713,22 +909,11 @@ class ControllerWithFilterInstance < PostsController
   around_filter YieldingFilter.new, :only => :raises_after
 end
 
-class ControllerWithFilterMethod < PostsController
-  class YieldingFilter < DefaultFilter
-    def filter(controller)
-      yield
-      raise After
-    end
-  end
-
-  around_filter YieldingFilter.new.method(:filter), :only => :raises_after
-end
-
 class ControllerWithProcFilter < PostsController
   around_filter(:only => :no_raise) do |c,b|
-    c.assigns['before'] = true
+    c.instance_variable_set(:"@before", true)
     b.call
-    c.assigns['after'] = true
+    c.instance_variable_set(:"@after", true)
   end
 end
 
@@ -770,18 +955,9 @@ class ControllerWithTwoLessFilters < ControllerWithAllTypesOfFilters
   skip_filter :after
 end
 
-class YieldingAroundFiltersTest < Test::Unit::TestCase
+class YieldingAroundFiltersTest < ActionController::TestCase
   include PostsController::AroundExceptions
 
-  def test_filters_registering
-    assert_equal 1, ControllerWithFilterMethod.filter_chain.size
-    assert_equal 1, ControllerWithFilterClass.filter_chain.size
-    assert_equal 1, ControllerWithFilterInstance.filter_chain.size
-    assert_equal 3, ControllerWithSymbolAsFilter.filter_chain.size
-    assert_equal 6, ControllerWithNestedFilters.filter_chain.size
-    assert_equal 4, ControllerWithAllTypesOfFilters.filter_chain.size
-  end
-
   def test_base
     controller = PostsController
     assert_nothing_raised { test_process(controller,'no_raise') }
@@ -810,16 +986,10 @@ class YieldingAroundFiltersTest < Test::Unit::TestCase
     assert_raise(After) { test_process(controller,'raises_after') }
   end
 
-  def test_with_method
-    controller = ControllerWithFilterMethod
-    assert_nothing_raised { test_process(controller,'no_raise') }
-    assert_raise(After) { test_process(controller,'raises_after') }
-  end
-
   def test_with_proc
-    controller = test_process(ControllerWithProcFilter,'no_raise')
-    assert controller.template.assigns['before']
-    assert controller.template.assigns['after']
+    test_process(ControllerWithProcFilter,'no_raise')
+    assert assigns['before']
+    assert assigns['after']
   end
 
   def test_nested_filters
@@ -839,13 +1009,13 @@ class YieldingAroundFiltersTest < Test::Unit::TestCase
   end
 
   def test_filter_order_with_all_filter_types
-    controller = test_process(ControllerWithAllTypesOfFilters,'no_raise')
-    assert_equal 'before around (before yield) around_again (before yield) around_again (after yield) around (after yield) after',controller.template.assigns['ran_filter'].join(' ')
+    test_process(ControllerWithAllTypesOfFilters,'no_raise')
+    assert_equal 'before around (before yield) around_again (before yield) around_again (after yield) after around (after yield)', assigns['ran_filter'].join(' ')
   end
 
   def test_filter_order_with_skip_filter_method
-    controller = test_process(ControllerWithTwoLessFilters,'no_raise')
-    assert_equal 'before around (before yield) around (after yield)',controller.template.assigns['ran_filter'].join(' ')
+    test_process(ControllerWithTwoLessFilters,'no_raise')
+    assert_equal 'before around (before yield) around (after yield)', assigns['ran_filter'].join(' ')
   end
 
   def test_first_filter_in_multiple_before_filter_chain_halts
@@ -853,7 +1023,6 @@ class YieldingAroundFiltersTest < Test::Unit::TestCase
     response = test_process(controller, 'fail_1')
     assert_equal ' ', response.body
     assert_equal 1, controller.instance_variable_get(:@try)
-    assert controller.instance_variable_get(:@before_filter_chain_aborted)
   end
 
   def test_second_filter_in_multiple_before_filter_chain_halts
@@ -861,7 +1030,6 @@ class YieldingAroundFiltersTest < Test::Unit::TestCase
     response = test_process(controller, 'fail_2')
     assert_equal ' ', response.body
     assert_equal 2, controller.instance_variable_get(:@try)
-    assert controller.instance_variable_get(:@before_filter_chain_aborted)
   end
 
   def test_last_filter_in_multiple_before_filter_chain_halts
@@ -869,13 +1037,11 @@ class YieldingAroundFiltersTest < Test::Unit::TestCase
     response = test_process(controller, 'fail_3')
     assert_equal ' ', response.body
     assert_equal 3, controller.instance_variable_get(:@try)
-    assert controller.instance_variable_get(:@before_filter_chain_aborted)
   end
 
   protected
     def test_process(controller, action = "show")
-      request = ActionController::TestRequest.new
-      request.action = action
-      controller.process(request, ActionController::TestResponse.new)
+      @controller = controller.is_a?(Class) ? controller.new : controller
+      process(action)
     end
 end
