@@ -1,10 +1,11 @@
+require File.expand_path('../../../load_paths', __FILE__)
 require 'active_model'
 
 class Person
   include ActiveModel::Conversion
   include ActiveModel::Validations
 
-  validates_presence_of :name
+  validates :name, presence: true
 
   attr_accessor :name
 
@@ -22,8 +23,8 @@ class Person
 end
 
 person1 = Person.new
-p person1.valid?
-person1.errors
+p person1.valid? # => false
+p person1.errors.messages # => {:name=>["can't be blank"]}
 
-person2 = Person.new(:name => "matz")
-p person2.valid?
+person2 = Person.new(name: 'matz')
+p person2.valid? # => true
