@@ -11,16 +11,18 @@ class TestDisconnectedAdapter < ActiveRecord::TestCase
   end
 
   def teardown
+    return if in_memory_db?
     spec = ActiveRecord::Base.connection_config
     ActiveRecord::Base.establish_connection(spec)
-    @connection = nil
   end
 
-  test "can't execute statements while disconnected" do
-    @connection.execute "SELECT count(*) from products"
-    @connection.disconnect!
-    assert_raises(ActiveRecord::StatementInvalid) do
+  unless in_memory_db?
+    test "can't execute statements while disconnected" do
       @connection.execute "SELECT count(*) from products"
+      @connection.disconnect!
+      assert_raises(ActiveRecord::StatementInvalid) do
+        @connection.execute "SELECT count(*) from products"
+      end
     end
   end
 end
