@@ -12,7 +12,7 @@ end
 
 class SpecialCategorization < ActiveRecord::Base
   self.table_name = 'categorizations'
-  default_scope where(:special => true)
+  default_scope { where(:special => true) }
 
   belongs_to :author
   belongs_to :category
