@@ -1,11 +1,11 @@
 require "isolation/abstract_unit"
 require "active_support/log_subscriber/test_helper"
 require "rack/test"
-require "mocha/setup"
 
 module ApplicationTests
   module RackTests
-    class LoggerTest < Test::Unit::TestCase
+    class LoggerTest < ActiveSupport::TestCase
+      include ActiveSupport::Testing::Isolation
       include ActiveSupport::LogSubscriber::TestHelper
       include Rack::Test::Methods
 
@@ -18,6 +18,7 @@ module ApplicationTests
       end
 
       def teardown
+        super
         teardown_app
       end
 
@@ -25,20 +26,26 @@ module ApplicationTests
         @logs ||= @logger.logged(:info).join("\n")
       end
 
-      test "logger logs proper HTTP verb and path" do
+      test "logger logs proper HTTP GET verb and path" do
         get "/blah"
         wait
         assert_match 'Started GET "/blah"', logs
       end
 
+      test "logger logs proper HTTP HEAD verb and path" do
+        head "/blah"
+        wait
+        assert_match 'Started HEAD "/blah"', logs
+      end
+
       test "logger logs HTTP verb override" do
-        post "/", :_method => 'put'
+        post "/", _method: 'put'
         wait
         assert_match 'Started PUT "/"', logs
       end
 
       test "logger logs HEAD requests" do
-        post "/", :_method => 'head'
+        post "/", _method: 'head'
         wait
         assert_match 'Started HEAD "/"', logs
       end
